@@ -1,15 +1,24 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__author__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-              "Alexis Thibault", "Tobias Van Baarsel"]
-__copyright__ = "Copyright 2020, Inria"
-__credits__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-               "Alexis Thibault", "Tobias Van Baarsel"]
-__license__ = "GPL 3.0"
-__version__ = "0.4"
-__email__ = "openwind-contact@inria.fr"
-__status__ = "Dev"
+# Copyright (C) 2019-2021, INRIA
+#
+# This file is part of Openwind.
+#
+# Openwind is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Openwind is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Openwind.  If not, see <https://www.gnu.org/licenses/>.
+#
+# For more informations about authors, see the CONTRIBUTORS file
 
 import numpy as np
 import scipy.special as sp
@@ -29,7 +38,8 @@ def losses_model(losses=True):
 
     Returns
     -------
-    losses_model : ThermoviscousModel
+    losses_model : :py:class:`ThermoviscousModel \
+    <openwind.continuous.thermoviscous_models.ThermoviscousModel>`
     """
 
     if isinstance(losses, ThermoviscousModel):
@@ -52,9 +62,36 @@ def losses_model(losses=True):
                      "{False, True (default), 'bessel' (same as True),"
                      " 'diffrepr', 'wl', 'keefe', 'minikeefe'}")
 
+def _sqrt_omegas(pipe, omegas_scaled):
+    """
+    Redimensionalization and square root of the frequency.
+
+    .. math::
+        \\sqrt{\\tilde{\\omega} T^{\\ast}}
+
+    with :math:`\\tilde{\\omega}` the scaled angular frequencies and
+    :math:`T^{\\ast}` the reference time used to scaled the frequencies.
+
+    Parameters
+    ----------
+    pipe: :py:class:`Pipe <openwind.continuous.pipe.Pipe>`
+    omegas_scaled: np.array (n,)
+        Dimensionless frequencies (scaled by the global time scaling).
+
+    Returns
+    -------
+    sqrt_omegas: np.array (n,1)
+        Redimensionalized root square of the frequencies, put in a shape that
+        is convenient for broadcasting operations to all frequencies.
+    """
+    omegas = omegas_scaled / pipe.get_scaling().get_time()
+    return np.sqrt(omegas)[:, np.newaxis]
+
 
 class ThermoviscousModel(ABC): # Abstract Base Class
-    """Which model to use for thermoviscous losses.
+    """
+
+    Abstract mother class for thermoviscous losses model.
 
     Assume that the model can be put under the form:
 
@@ -64,10 +101,10 @@ class ThermoviscousModel(ABC): # Abstract Base Class
 
     where:
 
-    - \(u\) is the scaled flow
-    - \(p\) is the scaled pressure
-    - \(x\) is the scaled position ( \(0 \\leq x \\leq 1\) ),
-    - \(\\omega\) is the scaled angular frequency,
+    - :math:`u` is the scaled flow
+    - :math:`p` is the scaled pressure
+    - :math:`x` is the scaled position ( :math:`0 \\leq x \\leq 1` ),
+    - :math:`\\omega` is the scaled angular frequency,
 
     And the coefficients
 
@@ -79,14 +116,15 @@ class ThermoviscousModel(ABC): # Abstract Base Class
 
     where :
 
-    - \(S(x)\) is the cross section area at the \(x\) position
-    - \(\\rho\) is the air density at the \(x\) position
-    - \(c\) is the sound celerity at the \(x\) position
-    - \(\\ell\) is the length of the pipe
-    - \(Y_t^{\\ast}, Z_v^{\\ast}\) are scaling coefficients
+    - :math:`S(x)` is the cross section area at the :math:`x` position
+    - :math:`\\rho` is the air density at the :math:`x` position
+    - :math:`c` is the sound celerity at the :math:`x` position
+    - :math:`\\ell` is the length of the pipe
+    - :math:`Y_t^{\\ast}, Z_v^{\\ast}` are scaling coefficients
 
     The aim of this class is to compute the two termes
-    \(\\text{Losses}(\\omega, x)\) at given positions and frequencies.
+    :math:`\\text{Losses}_u(\\omega, x)` and
+    :math:`\\text{Losses}_p(\\omega, x)` at given positions and frequencies.
 
 
     .. warning::
@@ -111,14 +149,19 @@ class ThermoviscousModel(ABC): # Abstract Base Class
 
         Parameters
         ----------
-        pipe : openwind.continuous.pipe.pipe
+        pipe : :py:class:`Pipe<openwind.continuous.pipe.Pipe>`
             The pipe considered
         x : float or array of float, within 0 and 1
             The normalized position(s) at which the coefficient is evaluated.
         omegas_scaled : float or array of float
             Scaled angular frequency(ies) at which the coefficient is evaluated
+
+        Returns
+        -------
+        array or float (same as x)
+            The Coefficient at the given frequency
         """
-        raise NotImplementedError()
+
 
     @staticmethod
     @abstractmethod
@@ -135,18 +178,23 @@ class ThermoviscousModel(ABC): # Abstract Base Class
 
         Parameters
         ----------
-        pipe : openwind.continuous.pipe.pipe
+        pipe : :py:class:`Pipe<openwind.continuous.pipe.Pipe>`
             The pipe considered
         x : float or array of float, within 0 and 1
             The normalized position(s) at which the coefficient is evaluated.
         omegas_scaled : float or array of float
             Scaled angular frequency(ies) at which the coefficient is evaluated
+
+        Returns
+        -------
+        array or float (same as x)
+            The Coefficient at the given frequency
         """
-        raise NotImplementedError()
 
     @staticmethod
     def get_diff_loss_flow(pipe, x, omegas_scaled, diff_index):
-        """Differential of flow coefficient w.r.t. diff_index.
+        """
+        Differential of flow coefficient w.r.t. diff_index.
 
         .. math::
             \\frac{\\partial \\text{Losses}_u(\\omega, x)}{\\partial m_i}
@@ -156,17 +204,22 @@ class ThermoviscousModel(ABC): # Abstract Base Class
 
         Parameters
         ----------
-        pipe : openwind.continuous.pipe.pipe
+        pipe : :py:class:`Pipe<openwind.continuous.pipe.Pipe>`
             The pipe considered
         x : float or array of float, within 0 and 1
             The normalized position(s) at which the coefficient is evaluated.
         omegas_scaled : float or array of float
             Scaled angular frequency(ies) at which the coefficient is evaluated
         diff_index : int
-            The index at which is stored the design parameter \(m_i\) in the
-            `openwind.design.design_parameter.OptimizationParameters`
+            The index at which is stored the design parameter :math:`m_i` in the
+            :py:class:`OptimizationParameters\
+            <openwind.design.design_parameter.OptimizationParameters>`
+
+        Returns
+        -------
+        array or float (same as x)
+            The differential of the coefficient at the given frequency
         """
-        raise NotImplementedError()
 
     @staticmethod
     def get_diff_loss_pressure(pipe, x, omegas_scaled, diff_index):
@@ -180,17 +233,22 @@ class ThermoviscousModel(ABC): # Abstract Base Class
 
         Parameters
         ----------
-        pipe : openwind.continuous.pipe.pipe
+        pipe : :py:class:`Pipe<openwind.continuous.pipe.Pipe>`
             The pipe considered
         x : float or array of float, within 0 and 1
             The normalized position(s) at which the coefficient is evaluated.
         omegas_scaled : float or array of float
             Scaled angular frequency(ies) at which the coefficient is evaluated
         diff_index : int
-            The index at which is stored the design parameter \(m_i\) in the
-            `openwind.design.design_parameter.OptimizationParameters`
+            The index at which is stored the design parameter :math:`m_i` in the
+            :py:class:`OptimizationParameters\
+            <openwind.design.design_parameter.OptimizationParameters>`
+
+        Returns
+        -------
+        array or float (same as x)
+            The differential of the coefficient at the given frequency
         """
-        raise NotImplementedError()
 
     def __repr__(self):
         return "<openwind.continuous.{}>".format(self.__class__.__name__)
@@ -230,44 +288,19 @@ class ThermoviscousLossless(ThermoviscousModel):
         return "<openwind.continuous.{}>: no losses".format(self.__class__.__name__)
 
 
-def _sqrt_omegas(pipe, omegas_scaled):
-    """
-    Redimensionalization and square root of the frequency.
-
-    .. math::
-        \\sqrt{\\tilde{\\omega} T^{\\ast}}
-
-    with \(\\tilde{\\omega}\) the scaled angular frequencies and \(T^{\\ast}\)
-    the reference time used to scaled the frequencies.
-
-    Parameters
-    ----------
-    pipe: openwind.continuous.Pipe
-    omegas_scaled: np.array (n,)
-        Dimensionless frequencies (scaled by the global time scaling).
-
-    Returns
-    -------
-    sqrt_omegas: np.array (n,1)
-        Redimensionalized frequencies, put in a shape that is
-        convenient for broadcasting operations to all frequencies.
-    """
-    omegas = omegas_scaled / pipe.get_scaling().get_time()
-    return np.sqrt(omegas)[:, np.newaxis]
-
-
-
 class ThermoviscousBessel(ThermoviscousModel):
-    """Bessel model for thermoviscous losses.
+    """
+    Bessel model for thermoviscous losses.
 
-    Model is given by the formulae (5.132 - 5.134) of [1].
+    Model is given by the formulae (5.132) to (5.134) of [Chaigne_ZK]_
+    (Chap.5.5, p.239).
     Note that the lossless part is included in the cited equations,
     but excluded from this computation.
 
     References
     ----------
-    [1] Chaigne, A., & Kergomard, J. (2016). Acoustics of musical instruments.
-        Springer New York.
+    .. [Chaigne_ZK] Chaigne, A., & Kergomard, J. (2016). "Acoustics of musical\
+        instruments". Springer New York.
     """
 
     def __str__(self):
@@ -340,16 +373,25 @@ class ThermoviscousBessel(ThermoviscousModel):
 
 
 class Keefe(ThermoviscousModel):
-    """Bessel model for Keefe losses.
+    """
+    Keefe losses model: 2nd order approximation of ZK losses.
 
-    Model is given in [1] .
+    Model is given in eq.(5.143) and (5.144) in [Chaigne_Keefe2]_
+    (Chap.5.5.3, p.242) or [Keefe2]_.
     Note that the lossless part is included in the cited equations,
     but excluded from this computation.
 
     References
     ----------
-    [1] Chaigne, A., & Kergomard, J. (2016). Acoustics of musical instruments.
-        Springer New York.
+    .. [Chaigne_Keefe2] Chaigne, A., & Kergomard, J. (2016). Acoustics of
+        musical instruments. Springer New York.
+
+    .. [Keefe2] Keefe, Douglas H. 1984. “Acoustical Wave Propagation in
+        Cylindrical Ducts: Transmission Line Parameter Approximations for
+        Isothermal and Nonisothermal Boundary Conditions.” The Journal of the
+        Acoustical Society of America 75 (1): 58–62.
+        https://doi.org/10.1121/1.390300.
+
     """
 
     def __str__(self):
@@ -429,16 +471,26 @@ class Keefe(ThermoviscousModel):
 
 
 class MiniKeefe(ThermoviscousModel):
-    """Bessel model for truncated Keefe losses.
+    """
+    1st order approximation of Keefe losses model (approximation of ZK losses).
+
+    Model is first order approximation of eq.(5.143) and (5.144) in
+    [Chaigne_Keefe1]_  (Chap.5.5.3, p.242) or [Keefe1]_.
 
-    Model is given in [1].
     Note that the lossless part is included in the cited equations,
     but excluded from this computation.
 
     References
     ----------
-    [1] Chaigne, A., & Kergomard, J. (2016). Acoustics of musical instruments.
-        Springer New York.
+    .. [Chaigne_Keefe1] Chaigne, A., & Kergomard, J. (2016). Acoustics of
+        musical instruments. Springer New York.
+
+    .. [Keefe1] Keefe, Douglas H. 1984. “Acoustical Wave Propagation in
+        Cylindrical Ducts: Transmission Line Parameter Approximations for
+        Isothermal and Nonisothermal Boundary Conditions.” The Journal of the
+        Acoustical Society of America 75 (1): 58–62.
+        https://doi.org/10.1121/1.390300.
+
     """
 
     def __wtr__(self):
@@ -472,6 +524,11 @@ class MiniKeefe(ThermoviscousModel):
         coef_loss_pressure = coef_pressure * (gamma - 1) * coefY
         return coef_loss_pressure
 
+    def get_diff_loss_flow(self, pipe, x, omegas_scaled, diff_index):
+        raise NotImplementedError()  # TODO ?
+
+    def get_diff_loss_pressure(self, pipe, x, omegas_scaled, diff_index):
+        raise NotImplementedError()  # TODO ?
 
 
 
@@ -545,18 +602,38 @@ def _get_diff_repr_coefs(name: str):
 
 
 class ThermoviscousDiffusiveRepresentation(ThermoviscousModel):
-    """Thermoviscous losses model with Diffusive Representation."""
+    """
+    Thermoviscous losses model with Diffusive Representation.
+
+    Approximates the Zwikker--Kosten model (ThermoviscousBessel) using a sum of
+    first-order rational fractions.
+    This model is suited to time-domain simulation.
+
+    More detail in [Thibault]_.
+
+    See Also
+    --------
+    :py:class:`TemporalLossyPipe <openwind.temporal.TemporalLossyPipe>`
+        The temporal lossy pipe, in which is implemented a temporal numerical
+        scheme of these losses.
+
+
+    Parameters
+    ----------
+    diff_repr_coefs : (float, float[], float[]) or str, optional
+        Coefficients (a_0, a_i, b_i) to put in the model,
+        or a string 'diffreprN' where N is the number of
+        additionnal variables. The default is 'diffrepr' corresponding to N=8.
+
+    References
+    ----------
+    .. [Thibault] Thibault, A. and Chabassier, J. "Dissipative time-domain 1D
+        model for viscothermal acoustic propagation in wind instruments".
+        To be published.
+
+    """
 
     def __init__(self, diff_repr_coefs='diffrepr'):
-        """
-        Parameters
-        ----------
-        pipe : Pipe on which this model is applied
-        diff_repr_coefs : (float, float[], float[]) or str
-            Coefficients (a_0, a_i, b_i) to put in the model,
-            or a string 'diffreprN' where N is the number of
-            additionnal variables.
-        """
         if isinstance(diff_repr_coefs, str):
             diff_repr_coefs = _get_diff_repr_coefs(diff_repr_coefs)
 
@@ -570,7 +647,13 @@ class ThermoviscousDiffusiveRepresentation(ThermoviscousModel):
         return "Diffusive Representation of losses"
 
     def get_number_of_dampers(self):
-        "Number of indices i used to count additionnal variables V_i or P_i."
+        """
+        Number of indices i used to count additionnal variables V_i or P_i.
+
+        Returns
+        -------
+        int
+        """
         return self._ai.shape[0]
 
     def get_loss_flow_at(self, pipe, x, omegas_scaled):
@@ -615,7 +698,23 @@ class ThermoviscousDiffusiveRepresentation(ThermoviscousModel):
         raise NotImplementedError()  # TODO ?
 
     def get_viscous_coefficients_at(self, pipe, x):
-        """Nondimensionalized coefficients R_0(x), R_i(x), L_i(x)."""
+        """
+        Nondimensionalized coefficients R_0(x), R_i(x), L_i(x).
+
+        Coefficients related to viscous effect. Used in temporal domain.
+
+        Parameters
+        ----------
+        pipe : :py:class:`Pipe<openwind.continuous.pipe.Pipe>`
+            The pipe considered
+        x : float or array of float, within 0 and 1
+            The normalized position(s) at which the coefficient is evaluated.
+
+        Returns
+        -------
+        r0, ri, li : array
+
+        """
         rhos, mus = pipe.get_physics().get_coefs(x, 'rho', 'mu')
         section = pipe.get_section_at(x)
         scaling_l = pipe.get_scaling_flow()
@@ -630,7 +729,23 @@ class ThermoviscousDiffusiveRepresentation(ThermoviscousModel):
         return r0, ri, li
 
     def get_thermal_coefficients_at(self, pipe, x):
-        """Nondimensionalized coefficients G_0(x), G_i(x), C_0(x), C_i(x)."""
+        """
+        Nondimensionalized coefficients G_0(x), G_i(x), C_0(x), C_i(x).
+
+        Coefficients related to thermal effect. Used in temporal domain.
+
+        Parameters
+        ----------
+        pipe : :py:class:`Pipe<openwind.continuous.pipe.Pipe>`
+            The pipe considered
+        x : float or array of float, within 0 and 1
+            The normalized position(s) at which the coefficient is evaluated.
+
+        Returns
+        -------
+        g0, gi, c0, ci : array
+
+        """
         rhos, kappas, Cp, gamma, cs = \
             pipe.get_physics().get_coefs(x, 'rho', 'kappa', 'Cp', 'gamma', 'c')
         section = pipe.get_section_at(x)
@@ -652,22 +767,25 @@ class ThermoviscousDiffusiveRepresentation(ThermoviscousModel):
 
 
 class WebsterLokshin(ThermoviscousModel):
-    """Webster-Lokshin model of boundary layer losses.
+    """
+    Webster-Lokshin model of boundary layer losses.
 
     Losses are applied to the pressure term only,
     and are expressed as a fractional derivative w.r.t. time.
-    Formulae are given in [1], eq. (16).
+    Formulae are given in [Helie_WL]_, eq. (16).
+    Their reformulation in the setting of openwind is given in [RR]_, eq. (52)
 
     -  No contribution of losses to the flow term: \
-        \( \\text{Losses}_u(\\omega, x)=0\)
+        :math:` \\text{Losses}_u(\\omega, x)=0`
     - Fractional derivative on the pressure.
 
     References
     ----------
-    [1] Hélie, Thomas and Hézard, Thomas and Mignot, Rémi and Matignon,
-        Denis. One-dimensional acoustic models of horns and comparison
-        with measurements. (2013) Acta Acustica united with Acustica,
-        vol. 99 (n° 6). pp. 960-974. ISSN 1610-1928
+    .. [Helie_WL] Hélie, Thomas and Hézard, Thomas and Mignot, Rémi and
+        Matignon,  Denis. "One-dimensional acoustic models of horns and
+        comparison with measurements". (2013) Acta Acustica united with
+        Acustica, vol. 99 (n° 6). pp. 960-974. ISSN 1610-1928
+    .. [RR] Alexis Thibault, Juliette Chabassier. Viscothermal models for wind musical instruments. [Research Report] RR-9356, Inria Bordeaux Sud-Ouest. 2020. ⟨hal-02917351⟩
     """
 
     def __repr__(self):
@@ -693,3 +811,9 @@ class WebsterLokshin(ThermoviscousModel):
         coef_pressure = pipe.get_coef_pressure_at(x)
         coef_loss_pressure = coef_pressure * coefY
         return coef_loss_pressure
+
+    def get_diff_loss_flow(self, pipe, x, omegas_scaled, diff_index):
+        raise NotImplementedError()  # TODO ?
+
+    def get_diff_loss_pressure(self, pipe, x, omegas_scaled, diff_index):
+        raise NotImplementedError()  # TODO ?
