@@ -1,16 +1,27 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__author__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-              "Alexis Thibault", "Tobias Van Baarsel"]
-__copyright__ = "Copyright 2020, Inria"
-__credits__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-               "Alexis Thibault", "Tobias Van Baarsel"]
-__license__ = "GPL 3.0"
-__version__ = "0.4"
-__email__ = "openwind-contact@inria.fr"
-__status__ = "Dev"
-"""Delayed evaluation and differentiation of parameters.
+# Copyright (C) 2019-2021, INRIA
+#
+# This file is part of Openwind.
+#
+# Openwind is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Openwind is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Openwind.  If not, see <https://www.gnu.org/licenses/>.
+#
+# For more informations about authors, see the CONTRIBUTORS file
+
+"""
+Delayed evaluation and differentiation of parameters.
 
 In the context of optimization, equation coefficients must be evaluated
 and differentiated for various values of parameters (such as tube length,
@@ -27,11 +38,11 @@ import numpy as np
 
 def eval_(params):
     """
-    Evaluate a list of DesignParameters.
+    Evaluate a list of :py:class:`DesignParameter<DesignParameter>`.
 
     Parameters
     ----------
-    params : list of DesignParameter
+    params : list of :py:class:`DesignParameter<DesignParameter>`
 
     Returns
     -------
@@ -43,11 +54,11 @@ def eval_(params):
 
 def diff_(params, diff_index):
     """
-    Differentiate a list of DesignParameters.
+    Differentiate a list of :py:class:`DesignParameter<DesignParameter>`.
 
     Parameters
     ----------
-    params : list of DesignParameter
+    params : list of :py:class:`DesignParameter<DesignParameter>`
     diff_index : int
         Index of the optimized parameter considered for the differentiation
 
@@ -60,48 +71,6 @@ def diff_(params, diff_index):
     """
     return [p.get_differential(diff_index) for p in params]
 
-def sigmoid(zeta):
-    """
-    Sigmoid function for the constraint range parameters.
-
-    Parameters
-    ----------
-    zeta : float
-        Real number
-
-    Returns
-    -------
-    float
-        The value of the sigmoid at zeta. The value is, by definition between 0
-        and 1
-
-    """
-    return 0.5 + np.arctan(zeta)/np.pi
-
-def diff_sigmoid(zeta):
-    """ Derivative of the sigmoid wr to zeta"""
-    return 1/(np.pi*(zeta**2 + 1))
-
-def inv_sigmoid(x):
-    """
-    The inverse of the sigmoid function.
-
-    Parameters
-    ----------
-    x : float
-        The value of the `sigmoid`. Must be between 0 and 1
-
-    Returns
-    -------
-    zeta : float
-        The coordinate at which the `sigmoid` equals x.
-
-    """
-    assert np.all(x < 1) and np.all(x > 0)
-    zeta = np.tan(np.pi*(x - 0.5))
-    return zeta
-
-
 
 class OptimizationParameters:
     """Manage the variable parameters for optimization.
@@ -116,43 +85,46 @@ class OptimizationParameters:
         The value associated to each parameter
     labels : list of string
         The label of each parameter
-    optimized : list of bool
+    active : list of bool
         If the parameter is include (True) or not (False) in the optimization
         process
-    geom_value : list of DesignParameter.get_value
+    geom_value : list of :py:meth:`get_value \
+        <openwind.design.design_parameter.DesignParameter.get_value>`
         The methods used to compute the geometric value of the parameters from
-        the value stored in the `OptimizationParameters.values` list.
+        the value stored in the :py:attr:`values<OptimizationParameters.values>` list.
+    bounds : list of tuple of 2 floats
+        The bounds for each parameter.
     """
 
+    MARGIN = 1e-7
+    """ margin added to the bounds to guarantee their respect"""
+
     def __init__(self):
         self.values = list()
-        self.labels = []
-        self.active = []
-        """list of `bool` : If the parameter is include (True) or not (False)\
-            in the optimization"""
-        self.geom_values = []
-        """list of `DesignParameter.get_value` : The methods used to compute\
-            the geometric value of the parameters from the value stored in the\
-            `OptimizationParameters.values` list."""
+        self.labels = list()
+        self.active = list()
+        self.geom_values = list()
+        self.bounds = list()
 
     def __str__(self):
-        msg = '{:20s}|\t{:15s}|\t{:15s}|\t{:6s}\n'.format('Labels',
-                                                          'Optim.Values',
-                                                          'Geom.Values',
-                                                          'Active')
-        msg += '-'*70 + '\n'
+        msg = ('{:20s}| {:>12s} || {:>10s} < {:>12s} < {:<10s} |'
+               ' {:>6s}\n'.format('Labels', 'Geom.Values', 'Min',
+                                  'Optim.Values', 'Max', 'Active'))
+        msg += '-'*85 + '\n'
         for k in range(len(self.labels)):
-            msg += ('{:20s}|\t{:+.7e} |\t{:+.7e} |'
-                    '\t{:}\n').format(self.labels[k], self.values[k],
-                                      self.get_geometric_values()[k],
-                                      self.active[k])
+            lb, ub = self.bounds[k]
+            msg += ('{:20s}| {:12.5e} || {:10.3g} < {:12.5e} < {:<10.3g} |'
+                    ' {:}\n').format(self.labels[k],
+                                     self.get_geometric_values()[k],
+                                     lb, self.values[k], ub, self.active[k])
         return msg
 
     def __repr__(self):
         return "<{class_}: {labels}>".format(class_=type(self).__name__,
-                                            labels=self.labels)
+                                             labels=self.labels)
 
-    def new_param(self, value, label, get_geom_values):
+    def new_param(self, value, label, get_geom_values,
+                  bounds=(-np.inf, np.inf)):
         """
         Add a new optimized parameter.
 
@@ -162,15 +134,18 @@ class OptimizationParameters:
             The initial value associated to the new parameter.
         label : str
             The label of the new parameter.
-        get_geom_values : DesignParameter.get_value
+        get_geom_values : :py:meth:`get_value \ <openwind.design.design_parameter.DesignParameter.get_value>`
             The method used to compute the geometric value of the parameter
             from its stored value.
+        bounds : tuple of two floats, optional
+            The bounds associated to the new parameters: (min, max).
+            The default value is no bounds: (-inf, inf).
 
         Returns
         -------
         new_index : int
             The index at which is stored this parameters in the lists of the
-            `OptimizationParameters` object.
+            py:class:`OptimizationParameters<OptimizationParameters>` object.
 
         """
         new_index = len(self.values)
@@ -178,6 +153,8 @@ class OptimizationParameters:
         self.labels.append(label)
         self.active.append(True)
         self.geom_values.append(get_geom_values)
+        self.bounds.append(bounds)
+
         return new_index
 
     def get_geometric_values(self):
@@ -193,9 +170,10 @@ class OptimizationParameters:
 
     def set_active_parameters(self, indices):
         """
-        Chose each stored parameters is include in the optimization process.
+        Include the designated parameters in the optimization process.
 
-        It modifies the `OptimizationParameters.active` attribute.
+        It modifies the :py:attr:`active<OptimizationParameters.active>`
+        attribute.
 
         Parameters
         ----------
@@ -213,7 +191,7 @@ class OptimizationParameters:
 
     def get_active_values(self):
         """
-        Return the value of the parameters included in the optim. process.
+        Return the value of the parameters included in the optimization process
 
         Returns
         -------
@@ -224,18 +202,31 @@ class OptimizationParameters:
                         zip(self.values, self.active) if optim]
         return optim_values
 
+    def get_active_bounds(self):
+        """
+        Return the bounds of the parameters included in the optim. process.
+
+        Returns
+        -------
+        bounds : list of tuple of float
+
+        """
+        bounds = [(bound[0] + self.MARGIN, bound[1] - self.MARGIN)
+                  for (bound, optim) in zip(self.bounds, self.active) if optim]
+        return bounds
+
     def set_active_values(self, new_values):
         """
         Modify the value of the parameters included in the optim. process.
 
-        It is typiccally done at each step of an optimization process.
+        It is typically done at each step of an optimization process.
 
         Parameters
         ----------
         new_values : list of float
             The list of the new values. Its length must correspond to the
             number of parameters included in the optimization process (True in
-            `OptimizationParameters.active`)
+            :py:attr:`active<OptimizationParameters.active>`)
 
         """
         values = np.array(self.values)
@@ -262,23 +253,24 @@ class OptimizationParameters:
         """
         Differentiate the parameter with respect to one parameter.
 
-        The result is typically 1 or 0
-        The optimization
-        variable designated by "diff_index"
+        .. math::
+            \\frac{\\partial \\zeta_k}{\\partial \\zeta_l}
+
+        The result is typically 1 or 0.
 
         Parameters
         ----------
         param_index : int
-            The parameter which is differentiate.
+            The parameter which is differentiate (:math:`k`).
         diff_index : int
             The index of the parameter w.r. to which the differentiation is
-            computed.
+            computed (:math:`l`).
 
         Returns
         -------
         dparam : float
-            The value of the differentiate: 1 if the two parameters correspond,
-            0 either.
+            The value of the differentiate: 1 if the two parameters correspond
+            (:math:`k=l`), 0 either.
         """
         indices_active = np.where(self.active)[0]
         if param_index == indices_active[diff_index]:
@@ -293,9 +285,14 @@ class OptimizationParameters:
 
 
 class DesignParameter(ABC):
-    """Parameter of a DesignShape.
+    """
+    A geometric parameter used to design an instrument.
 
-    Parent class of the different kinds of parameters."""
+    It is used to parameterized a :py:class:`DesignShape<openwind.design.DesignShape>`,
+    or to specify the location of a side hole on the main bore.
+
+    Parent class of the different kinds of parameters.
+    """
 
     @abstractmethod
     def get_value(self):
@@ -309,7 +306,8 @@ class DesignParameter(ABC):
         Parameters
         ----------
         diff_index : int
-            Index of the active parameter of the `OptimizationParameters`
+            Index of the active parameter of the :py:class:`OptimizationParameters \
+            <openwind.design.design_parameter.OptimizationParameters>`
             considered for the differentiation.
 
         Return
@@ -324,7 +322,7 @@ class DesignParameter(ABC):
         """Variable status (Fixed: False, else: True)."""
 
     def __str__(self):
-        return "{}".format(self.get_value())
+        return "{:<7.5g}".format(self.get_value())
 
     def __repr__(self):
         return '{label}:{class_}({value})'.format(label=self.label,
@@ -333,21 +331,19 @@ class DesignParameter(ABC):
 
 
 class FixedParameter(DesignParameter):
-    """Parameter with a constant value.
+    """
+    Parameter with a constant value.
 
     Parameters
     -----------
     value : float
         The geometric value of the parameter
     label : str, optional
-        The parameter's name. The default is None.
+        The parameter's name. The default is ''.
 
-    Attributes
-    ----------
-    label
     """
 
-    def __init__(self, value, label=None):
+    def __init__(self, value, label=''):
         self._value = value
         self.label = label
 
@@ -361,36 +357,56 @@ class FixedParameter(DesignParameter):
         return False
 
 
-
 class VariableParameter(DesignParameter):
     """
-    Simple variable parameter.
+    Variable geometric design parameter.
+
+    Parameter the value of which can be possibly modify after instanciation,
+    for example during an optimization process. The use of this class is
+    coupled with the :py:class:`OptimizationParameters \
+    <openwind.design.design_parameter.OptimizationParameters>`.
+
+    The geometric value equals the stored optimized value. This value is
+    bounded such as:
+
+    .. math::
+        x_{min}<x<x_{max}
 
-    The geometric value equals the stored optimized value.
-    Without external constraint the geometric value is a real (positive or
-    negative).
 
     Parameters
     ----------
     value : float
         The initial geometric value of the parameter
-    optim_params : OptimizationParameters
+    optim_params : :py:class:`OptimizationParameters \
+    <openwind.design.design_parameter.OptimizationParameters>`
         The object where is stored the variable value
     label : str, optional
-        The name of the parameter. the default is None.
+        The name of the parameter. the default is ''.
+    bounds : list of two float
+        The boundaries :math:`x_{min}` and :math:`x_{max}` of the authorized
+        range for this parameter. The default is no bounds: `(-inf, inf)`
 
     Attributes
     ----------
     label : str
+        The parameter name.
     index : int
         The position at which is stored this parmeter in the
-        `OptimizationParameters`
+        :py:class:`OptimizationParameters`
     """
 
-    def __init__(self, value, optim_params, label=None):
+    def __init__(self, value, optim_params, label='',
+                 bounds=(-np.inf, np.inf)):
         self._optim_params = optim_params
-        self.index = optim_params.new_param(value, label, self.get_value)
         self.label = label
+        if bounds[0] > bounds[1]:
+            raise ValueError('The low bound {} must be smaller than the upper'
+                             ' bound {}'.format(bounds[0], bounds[1]))
+        if (value <= bounds[0] or value >= bounds[1]):
+            raise ValueError('The initial value is not inside the authorized '
+                             'range.')
+        self.index = optim_params.new_param(value, label, self.get_value,
+                                            bounds)
 
     def get_value(self):
         return self._optim_params.get_param(self.index)
@@ -402,190 +418,71 @@ class VariableParameter(DesignParameter):
         return True
 
     def __str__(self):
-        return "~{}".format(self.get_value())
+        bounds = self._optim_params.bounds[self.index]
+        msg = "~{:>.5g}".format(self.get_value())
+        if not all(np.isinf(bounds)):
+            msg = "{}<".format(bounds[0]) + msg
+            if not np.isinf(bounds[1]):
+                msg += "<{}".format(bounds[1])
+        return msg
 
-class VariableParameterSquare(DesignParameter):
-    """
-    Variable parameters higher than a minimal value.
-
-    To assure that the geometric value is higher than a minimal value, it is
-    defined
-
-    .. math::
-        x = \\zeta^2 + x_{min}
-
-    where \(\\zeta\) is the real number (positive or negative) which is stored
-    and optimized.
-
-    By this way, it is assure that \( x \\geq x_{min}\)
-
-    Parameters
-    ----------
-    value : float
-        The initial geometric value of the parameter
-    optim_params : OptimizationParameters
-        The object where is stored the variable value
-    min_value : float
-        The \(x_{min}\). It is forced to be higher than 1e-10 to avoid
-        numerical problem.
-    label : str, optional
-        The name of the parameter. the default is None.
-
-    Attributes
-    ----------
-    label : str
-    index : int
-        The position at which is stored this parmeter in the
-        `OptimizationParameters`
-    """
-
-    def __init__(self, value, optim_params, min_value, label=None):
-        self._optim_params = optim_params
-        self.label = label
-        self._min_value = np.max([min_value, 1e-10])
-        if value <= self._min_value:
-            raise ValueError('The initial value must be higher that the '
-                             'minimal value.')
-        zeta = np.sqrt(value - self._min_value)
-        # zeta = np.log(value - self._min_value)
-        self.index = optim_params.new_param(zeta, label, self.get_value)
-
-    def get_value(self):
-        zeta = self._optim_params.get_param(self.index)
-        value = zeta**2 + self._min_value
-        # value = np.exp(zeta) + self._min_value
-        return value
-
-    def get_differential(self, diff_index):
-        d_zeta = self._optim_params.diff_param(self.index, diff_index)
-        zeta = self._optim_params.get_param(self.index)
-        return 2*zeta*d_zeta  # d_zeta*np.exp(zeta) #
-
-    def is_variable(self):
-        return True
-
-    def __str__(self):
-        return "{}<~{}".format(self._min_value, self.get_value())
-
-class VariableParameterLimitedRange(DesignParameter):
-    """
-    Variable parameter limited to an authorized range.
-
-    The parameters is defined as:
-
-    .. math::
-        x = (x_{min} - x_{max})\\Sigma(\\zeta) + x_{min}
-
-    with \(\\zeta\) an auxiliary real number (positive or negative) stored
-    and optimized. The function \(0<\\Sigma(\\zeta)<1\) is the `sigmoid`..
-    By this way, it is assure that \(x \\in ]x_{min} - x_{max}[\)
-
-    Parameters
-    ----------
-    value : float
-        The initial geometric value of the parameter
-    optim_params : OptimizationParameters
-        The object where is stored the variable value
-    authorized_range :  list of two float
-        The boundaries \(x_{min}\) and \(x_{max}\) of the authorized range for
-        this parameter.
-    label : str, optional
-        The name of the parameter. the default is None.
-
-    Attributes
-    ----------
-    label : str
-    index : int
-        The position at which is stored this parmeter in the
-        `OptimizationParameters`
-    """
-
-    def __init__(self, value, optim_params, authorized_range, label=None):
-        self._optim_params = optim_params
-        self.label = label
-        self._authorized_range = authorized_range
-        if (value <= np.min(authorized_range)
-            or value >= np.max(authorized_range)):
-            raise ValueError('The initial value is not inside the authorized '
-                             'range.')
-        zeta = self._get_auxiliary_var(value)
-        self.index = optim_params.new_param(zeta, label, self.get_value)
-
-    def _get_auxiliary_var(self, value):
-        min_auth = np.min(self._authorized_range)
-        max_auth = np.max(self._authorized_range)
-        zeta = inv_sigmoid((value - min_auth)/(max_auth - min_auth))
-        return zeta
-
-    def get_value(self):
-        min_auth = np.min(self._authorized_range)
-        max_auth = np.max(self._authorized_range)
-        zeta = self._optim_params.get_param(self.index)
-        value = (max_auth - min_auth)*sigmoid(zeta) + min_auth
-        return value
-
-    def get_differential(self, diff_index):
-        min_auth = np.min(self._authorized_range)
-        max_auth = np.max(self._authorized_range)
-        d_zeta = self._optim_params.diff_param(self.index, diff_index)
-        zeta = self._optim_params.get_param(self.index)
-        return (max_auth - min_auth) * d_zeta*diff_sigmoid(zeta)
-
-    def is_variable(self):
-        return True
-
-    def __str__(self):
-        return "{}<~{}<{}".format(np.min(self._authorized_range),
-                                  self.get_value(),
-                                  np.max(self._authorized_range))
 
 class VariableHolePosition(DesignParameter):
     """
     Variable hole position defined relatively on the main bore pipe.
 
-    The parameters is defined as:
+    The location of the hole :math:`x_{hole}` is defined as:
 
     .. math::
-        x = (x_{0} - x_{1})\\Sigma(\\zeta) + x_{0}
+        \\begin{align}
+        x_{hole} & =  (x_1 - x_0) \\zeta + x_0 \\\\
+        0 & \\leq \\zeta \\leq 1
+        \\end{align}
 
-    with \(\\zeta\) an auxiliary real number (positive or negative) stored
-    and optimized, \(x_{0},x_{1}\) the boundaries of the main bore pipe where
-    is placed the considered hole. The function \(0<\\Sigma(\\zeta)<1\) is the
-    `sigmoid`. By this way, it is assure that the hole stays on the same pipe:
-    \( x \\in ]x_{0} - x_{1}[\)]
+    with :math:`\\zeta` an auxiliary parameter bounded between 0 and 1,
+    :math:`(x_{0},x_{1})` the boundaries of the main bore pipe
+    where is placed the considered hole. It's assure that:
+    :math:`x_0 \\leq x_{hole} \\leq x_1`.
+
+    .. warning::
+        The stocked and optimized value is :math:`\\zeta` and not \
+        :math:`x_{hole}`
 
     Parameters
     ----------
     init_value : float
         The initial geometric value of the parameter
-    optim_params : OptimizationParameters
+    optim_params : :py:class:`OptimizationParameters \
+    <openwind.design.design_parameter.OptimizationParameters>`
         The object where is stored the variable value
-    main_bore_shape :  openwind.design.design_shape.DesignShape
+    main_bore_shape : :py:class:`DesignShape <openwind.design.DesignShape>`
         The shape of the pipe where is located the hole.
     label : str, optional
-        The name of the parameter. the default is None.
+        The name of the parameter. the default is ''.
 
     Attributes
     ----------
     label : str
+        The parameter name.
     index : int
         The position at which is stored this parmeter in the
-        `OptimizationParameters`
+        :py:class:`OptimizationParameters \
+        <openwind.design.design_parameter.OptimizationParameters>`
     """
-    def __init__(self, init_value, optim_params, main_bore_shape, label=None):
+
+    def __init__(self, init_value, optim_params, main_bore_shape, label=''):
         self._optim_params = optim_params
         self.label = label
         self._main_bore_shape = main_bore_shape
         norm_position = main_bore_shape.get_xnorm_from_position(init_value)
-        zeta = inv_sigmoid(norm_position)
-        self.index = optim_params.new_param(zeta, label, self.get_value)
+        self.index = optim_params.new_param(norm_position, label,
+                                            self.get_value, (0, 1))
 
     def is_variable(self):
         return True
 
     def get_value(self):
-        zeta = self._optim_params.get_param(self.index)
-        x_norm = sigmoid(zeta)
+        x_norm = self._optim_params.get_param(self.index)
         value = self._main_bore_shape.get_position_from_xnorm(x_norm)
         return value
 
@@ -593,8 +490,8 @@ class VariableHolePosition(DesignParameter):
         d_zeta = self._optim_params.diff_param(self.index, diff_index)
         zeta = self._optim_params.get_param(self.index)
 
-        x_norm = sigmoid(zeta)
-        dx_norm = d_zeta * diff_sigmoid(zeta)
+        x_norm = zeta
+        dx_norm = d_zeta
 
         Xmin, Xmax = eval_(self._main_bore_shape.get_endpoints_position())
         d_position = (self._main_bore_shape
@@ -602,36 +499,44 @@ class VariableHolePosition(DesignParameter):
         return dx_norm*(Xmax - Xmin) + d_position
 
     def __str__(self):
-        return "~{}%".format(self.get_value())
+        return "~{:>.5g}%".format(self.get_value())
 
 
 class VariableHoleRadius(DesignParameter):
     """
     Variable hole radius defined relatively to the main bore pipe radius.
 
-    The parameters is defined as:
+    The hole raidus :math:`r_{hole}` is defined as:
 
     .. math::
-        x = R_{main} \\Sigma(\\zeta)
+        \\begin{align}
+        r_{hole} & = r_{main} \\zeta \\\\
+        0 & \\leq \\zeta \\leq 1
+        \\end{align}
 
-    with \(\\zeta\) an auxiliary real number (positive or negative) stored
-    and optimized, \(R_{main}\) the radius of the main bore pipe at the position of
-    the considered hole. The function \(0<\\Sigma(\\zeta)<1\) is the `sigmoid`.
-    By this way, it is assure that the hole keeps a radius smaller than the one
-    of the main pipe: \(x \\leq R_{main}\)
+    with :math:`\\zeta` an auxiliary parameter bounded between 0 and 1,
+    :math:`r_{main}` the radius of the main bore pipe at the
+    position of the considered hole. It's assure that:
+    :math:`r_{hole} \\leq r_{main}`
+
+    .. warning::
+        The stocked and optimized value is :math:`\\zeta` and not \
+        :math:`r_{hole}`
 
     Parameters
     ----------
     init_value : float
         The initial geometric value of the parameter
-    optim_params : OptimizationParameters
+    optim_params : :py:class:`OptimizationParameters \
+    <openwind.design.design_parameter.OptimizationParameters>`
         The object where is stored the variable value
-    main_bore_shape :  openwind.design.design_shape.DesignShape
+    main_bore_shape :  :py:class:`DesignShape <openwind.design.DesignShape>`
         The shape of the pipe where is located the hole.
-    hole_position : DesignParameter
+    hole_position : :py:class:`DesignParameter \
+    <openwind.design.design_parameter.DesignParameter>`
         The position of the hole.
     label : str, optional
-        The name of the parameter. the default is None.
+        The name of the parameter. the default is ''.
 
     Attributes
     ----------
@@ -642,7 +547,7 @@ class VariableHoleRadius(DesignParameter):
     """
 
     def __init__(self, init_value, optim_params, main_bore_shape,
-                 hole_position, label=None):
+                 hole_position, label=''):
         self._optim_params = optim_params
         self.label = label
         self._hole_position = hole_position
@@ -650,8 +555,10 @@ class VariableHoleRadius(DesignParameter):
 
         x_norm = self.__get_x_norm()
         radius_main_pipe = main_bore_shape.get_radius_at(x_norm)
-        zeta = inv_sigmoid(init_value/radius_main_pipe)
-        self.index = optim_params.new_param(zeta, label, self.get_value)
+        # zeta = inv_sigmoid(init_value/radius_main_pipe)
+        zeta = init_value/radius_main_pipe
+        self.index = optim_params.new_param(zeta, label, self.get_value,
+                                            (0, 1))
 
     def is_variable(self):
         return True
@@ -660,7 +567,7 @@ class VariableHoleRadius(DesignParameter):
         zeta = self._optim_params.get_param(self.index)
         x_norm = self.__get_x_norm()
         radius_main_pipe = self._main_bore_shape.get_radius_at(x_norm)
-        value = radius_main_pipe * sigmoid(zeta)
+        value = radius_main_pipe * zeta
         return value
 
     def __get_x_norm(self):
@@ -684,13 +591,12 @@ class VariableHoleRadius(DesignParameter):
         r_pipe = self._main_bore_shape.get_radius_at(x_norm)
 
         dx_norm = self.__get_diff_x_norm(diff_index)
-        dr_pipe_dx_norm = self._main_bore_shape.get_diff_shape_wr_x_norm(x_norm)
+        dr_pipe_dx_norm = self._main_bore_shape.diff_radius_wr_x_norm(x_norm)
         dr_pipe_diff_index = (self._main_bore_shape
                               .get_diff_radius_at(x_norm, diff_index))
-
-        return (d_zeta*diff_sigmoid(zeta)*r_pipe
-                + sigmoid(zeta)*dr_pipe_dx_norm*dx_norm
-                + sigmoid(zeta)*dr_pipe_diff_index)
+        return (d_zeta*r_pipe
+                + zeta*dr_pipe_dx_norm*dx_norm
+                + zeta*dr_pipe_diff_index)
 
     def __str__(self):
-        return "~{}%".format(self.get_value())
+        return "~{:>.5g}%".format(self.get_value())
