@@ -1,57 +1,61 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__author__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-              "Alexis Thibault", "Tobias Van Baarsel"]
-__copyright__ = "Copyright 2020, Inria"
-__credits__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-               "Alexis Thibault", "Tobias Van Baarsel"]
-__license__ = "GPL 3.0"
-__version__ = "0.4"
-__email__ = "openwind-contact@inria.fr"
-__status__ = "Dev"
+# Copyright (C) 2019-2021, INRIA
+#
+# This file is part of Openwind.
+#
+# Openwind is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Openwind is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Openwind.  If not, see <https://www.gnu.org/licenses/>.
+#
+# For more informations about authors, see the CONTRIBUTORS file
 
 
 """
 Parse instrument geometry and fingering into OpenWind classes.
 """
-import collections
+
 import warnings
-
-try:
-    from tabulate import tabulate
-except ImportError:
-    # Backup (less-)pretty-printing function for when package is missing
-    def tabulate(data, col_names):
-        """ Adjust column width in a string. """
-        data = [col_names] + data
-        data = [[str(d) for d in row] for row in data]
-        maxlength = max(max(len(d) for d in row) for row in data)
-        format_per_element = "{:>%d}" % (maxlength+1)
-        format_per_row = format_per_element * len(data[0])
-        rows = [format_per_row.format(*row) for row in data]
-        return '\n'.join(rows)
-
 import numpy as np
 import matplotlib.pyplot as plt
 
 from openwind.design import (FixedParameter, OptimizationParameters,
-                             VariableParameter, VariableParameterSquare,
-                             VariableParameterLimitedRange,
+                             VariableParameter,
                              VariableHolePosition, VariableHoleRadius)
 from openwind.design import Spline, Cone, Bessel, Circle, Exponential
 from openwind.technical import FingeringChart
+from openwind.technical.fingering_chart import tabulate
 
-__author__ = ("Guillaume Castera, Juliette Chabassier, Augustin Ernoult,"
-              " Alexis Thibault, Robin Tournemenne")
-__copyright__ = "Copyright 2019, Inria"
-__credits__ = ["Guillaume Castera", "Juliette Chabassier", "Augustin Ernoult",
-               "Alexis Thibault", "Robin Tournemenne"]
-__license__ = "GPL 3.0"
-__version__ = "2.0"
-__maintainer__ = "Juliette Chabassier"
-__email__ = "juliette.chabassier@inria.fr"
-__status__ = "Dev"
+
+class Hole:
+    """
+    Hole with its shape and location on the main bore.
+
+    Attributes
+    ----------
+    shape : :py:class:`DesignShape <openwind.design.design_shape.DesignShape>`
+        The shape of the chimney.
+    position : :py:class:`DesignParameter \
+    <openwind.design.design_parameter.DesignParameter>`
+        The position of the Hole on the main bore.
+    label : string
+        The name of the hole.
+    """
+
+    def __init__(self, shape, position, label):
+        self.shape = shape
+        self.position = position
+        self.label = label
 
 
 class InstrumentGeometry:
@@ -60,62 +64,20 @@ class InstrumentGeometry:
 
     Create an instrument with the bore shape and holes given by the
     geometry described in the files.
-    Up to three files can be provided :
 
-    * main bore geometry
-    * side holes
-    * fingering chart
-
-
-    Attributes
+    Parameters
     ----------
-    main_bore_shapes: list[ openwind.design.design_shape.DesignShape ]
-        The description of the main bore
-    holes: list[ openwind.technical.instrument_geometry.InstrumentGeometry.Hole ]
-        The description of the tone holes
-    fingering_chart: openwind.technical.fingering_chart.FingeringChart
-        The fingerings of the various notes.
-    optim_params: openwind.design.design_parameter.OptimizationParameters
-        Used only for inverse problem. Organized the variable parameters which
-        are modified during the optimization process.
+    files : str or list
+        filename or list of data respecting the file format. Up to three files\
+        can be provided respecting the followind order :
 
-    Methods
-    -------
-    `plot_InstrumentGeometry(figure=None, note=None, **kwargs)` :
-        Plot the instrument.
+        1. the file (or list) containing the main bore geometry
+        2. the file (or list) with the side holes geometry and location
+        3. the file (or list) indicating the fingering chart
 
-    `print_files(generic_name, extension='.txt')` :
-        Print the files corresponding to this InstrumentGeometry.
 
-    `get_main_bore_radius_at(position)` :
-        Return the radius of the main bore at given position.
 
-    `get_bore_list()` :
-        Return the lists of data of the geometry.
-
-    `get_main_bore_length()` :
-        Return the total length of the main bore.
-
-    `print_main_bore_shape(filename=None)` :
-        Print the main bore shape.
-
-    `print_holes(filename=None)` :
-        Print the holes information.
-
-    See Also:
-    --------
-    `openwind.design.design_parameter.DesignParameter` : Mother class of design
-    parameters
-
-    `openwind.design.design_shape.DesignShape` : Mother class of design shapes
-
-    `openwind.design.design_parameter.OptimizationParameters`
-
-    `openwind.technical.fingering_chart.FingeringChart`
-
-    Structure of the files
-    -----------------------
-    -----------------------
+    **Structure of the files**
 
     In all of the files, data are separated by whitespace.
     Comments start with a `#` and extend to the end of the line.
@@ -127,8 +89,7 @@ class InstrumentGeometry:
         >>> InstrumentGeometry([[0.0, 2e-3], [0.2, 0.01]])
         <openwind.InstrumentGeometry at ...>
 
-
-    * Main bore geometry file *
+    *Main bore geometry file*
 
     The instrument bore is assumed to be aligned along one axis `x`.
     Abscissae `x` are absolute, with `x = 0` corresponding to the
@@ -137,24 +98,22 @@ class InstrumentGeometry:
     Each line describes one section of the instrument,
     which can be of one of several basic shapes.
     Measurements are given in meters.
-
     The line must be of the form, either:
 
-    * `x1 x2 r1 r2 type [param...]`
-        where
+    - ``x1 x2 r1 r2 type [param...]``, where:
 
-        * x1, x2 are the beginning and end abscissae of the part,
-        * r1, r2 are the radii at the beginning and end,
-        * `type` is one of `{'linear', 'spline', 'circle', 'exponential', 'bessel'}`,
-        * `param...` are the parameters of the shape, if necessary:
-            - radius of the `circle`
-            - `alpha` parameter of the `bessel` function
-            - `x_i... r_i...` internal points of the `spline`
+        - ``x1, x2`` are the beginning and end abscissae of the part,
+        - ``r1, r2`` are the radii at the beginning and end,
+        - ``type`` is one of ``{'linear', 'spline', 'circle', 'exponential', 'bessel'}``
+        - ``[param ...]`` are the parameters of the shape, if necessary:
 
-    * `x r`
-        in which case the radius is assumed to evolve linearly from the
+            - radius of the 'circle'
+            - ``alpha`` parameter of the 'bessel' function
+            - ``x_i... r_i...`` internal points of the 'spline'
+    - ``x r``: in which case the radius is assumed to evolve linearly from the\
         last specified point.
 
+
     .. code-block:: shell
 
         # This is my favorite instrument
@@ -167,15 +126,17 @@ class InstrumentGeometry:
 
 
 
-    * Holes file *
+    *Holes file*
 
     A new format for holes file has been introduced:
     First line of the file contains column titles,
     the following lines contain the data.
 
     Possible column names are : 'label', 'position', 'chimney', 'radius', 'type'.
+
     Columns 'position', 'chimney', 'radius' are mandatory, 'label' and 'type'
     are optional.
+
     If 'label' is provided, each hole will be labeled by the given name.
     Currently the only supported 'type' is 'linear'.
 
@@ -188,17 +149,14 @@ class InstrumentGeometry:
         ...
 
 
-    Alternate (old) format : `x l r type`.
+    Alternate (old) format : ``x l r type``.
     This format is deprecated and may give inconsistent hole labels.
 
-    * Fingering chart file *
+    *Fingering chart file*
 
-    First line of the file must be:
-        `label [note_name...]`
-    The following lines are:
-        `hole_name ['x' or 'o' ...]`
-        specifying for each note, whether this hole is
-        open ('o') or closed ('x').
+    First line of the file must be: ``label [note_name...]``.
+    The following lines are: ``hole_name ['x' or 'o' ...]`` specifying for \
+        each note, whether this hole is open ('o') or closed ('x').
 
     .. code-block:: shell
 
@@ -209,7 +167,7 @@ class InstrumentGeometry:
         ...
 
 
-    * Type of design parameters for the inversion *
+    **Type of design parameters for the inversion**
 
     For the inversion, it is necessary to indicate which parameters are fixed,
     and which parameters is variable and can be modified by the algorithm. This
@@ -218,36 +176,50 @@ class InstrumentGeometry:
 
     Several types of variable parameters can be used:
 
-    * `0.01`: The numerical value only is a fixed parameters.
-            A ``FixedParameter`` object is instantiated.
+    - '0.01': The numerical value only is a fixed parameters. \
+        A :class:`FixedParameter` object is instantiated.
+    - '~0.01': The '~' indicates a variable parameters (unlimited range). \
+        A :class:`VariableParameter` object is instantiated without bound.
+    - '1e-3<~0.01': The range of the variable parameter is limited by a low \
+        bound. A :class:`VariableParameter` object is instantiated with lower \
+        bound only.
+    - '1e-3<~0.01<1e-1': The range of the variable parameter is limited by low\
+        and upper bounds. A :class:`VariableParameter` object is instantiated \
+        with lower and upper bounds.
+    - '~0.01%': Only valid for hole position and hole radius! \
+        The parameter is defined relatively to the main bore shape on which \
+        the hole is placed. A :py:class:`VariableHolePosition \
+        <openwind.design.design_parameter.VariableHolePosition>` or \
+        :py:class:`VariableHoleRadius \
+        <openwind.design.design_parameter.VariableHoleRadius>` object is instantiated.
 
-    * `~0.01`: The '~' indicates a variable parameters (unlimited range).
-            A ``VariableParameter`` object is instantiated.
+    See Also
+    --------
+    :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` :
+        Mother class of design parameters
+    :py:class:`DesignShape <openwind.design.design_shape.DesignShape>` :
+        Mother class of design shapes
 
-    * `1e-3<~0.01`: The range of the variable parameter is limited by a low bound.
-            A ``VariableParameterSquare`` object is instantiated.
 
-    * `1e-3<~0.01<1e-1`: The range of the variable parameter is limited by low and upper bounds.
-            A ``VariableParameterLimitedRange`` object is instantiated.
+    Attributes
+    ----------
+    main_bore_shapes: list[ :py:class:`DesignShape \
+    <openwind.design.design_shape.DesignShape>` ]
+        The description of the main bore
+    holes: list[ :py:class:`Hole \
+    <openwind.technical.instrument_geometry.Hole>` ]
+        The description of the tone holes
+    fingering_chart: :py:class:`FingeringChart \
+    <openwind.technical.fingering_chart.FingeringChart>`
+        The fingerings of the various notes.
+    optim_params: :py:class:`OptimizationParameters \
+    <openwind.design.design_parameter.OptimizationParameters>`
+        Used only for inverse problem. Organized the variable parameters which
+        are modified during the optimization process.
 
-    * `~0.01%`: Only valid for hole position and hole radius!
-            The parameter is defined relatively to the main bore shape on which
-            the hole is placed.
-            A `VariableHolePosition` or `VariableHoleRadius` object is instantiated.
     """
 
-    Hole = collections.namedtuple('Hole', ['shape', 'position', 'label'])
-    # Collection used to instanciate a subclass `Hole`.
-    #
-    # Attributes
-    # ----------
-    # shape : openwind.design.design_shape.DesignShape
-    #     The shape of the chimney.
-    # position : openwind.design.design_parameter.DesignParameter
-    #     The position of the Hole on the main bore.
-    # label : string
-    #     The name of the hole.
-
     def __init__(self, *files):
         self.optim_params = OptimizationParameters()
         self._create_main_bore_shapes(files[0])
@@ -261,12 +233,10 @@ class InstrumentGeometry:
         else:
             self.fingering_chart = self._create_fingering_chart(None)
 
-
     def __repr__(self):
         return '<openwind.InstrumentGeometry({} main bore parts, {} holes, ' \
                '{})>'.format(len(self.main_bore_shapes),
-                                            len(self.holes),
-                                            repr(self.fingering_chart))
+                             len(self.holes), repr(self.fingering_chart))
 
     def __str__(self):
         return(('-'*30 + '\nMain Bore:\n{} \n' + '-'*30 + '\nHoles:\n{} \n' +
@@ -274,10 +244,7 @@ class InstrumentGeometry:
                .format(self.print_main_bore_shape(), self.print_holes(),
                        self.fingering_chart))
 
-    def get_hole_labels(self):
-        return [hole.label for hole in self.holes]
-
-# %% creation of lists of shape and fingering chart
+    # %% creation of lists of shape and fingering chart
     def _create_main_bore_shapes(self, data):
         """
         Construct the "DesignShapes" of the main bore from the data.
@@ -292,12 +259,13 @@ class InstrumentGeometry:
 
         Attributes
         ----------
-        main_bore_shapes : list of [ openwind.design.design_shape.DesignShape ]
+        main_bore_shapes : list of [ :py:class:`DesignShape \ <openwind.design.design_shape.DesignShape>` ]
             list of DesignShape
 
         """
         self.main_bore_shapes = list()
-        """A list of `openwind.design.design_shape.DesignShape` which describe
+        """A list of :py:class:`DesignShape \
+        <openwind.design.design_shape.DesignShape>` which describe
         the main bore of the instrument."""
         main_bore_list = self._interpret_data(data)
         Xlast = None
@@ -359,7 +327,8 @@ class InstrumentGeometry:
 
     def _create_fingering_chart(self, data):
         """
-        Construct the ```FingeringChart``` from the data.
+        Construct the :py:class:`FingeringChart \
+        <openwind.technical.fingering_chart.FingeringChart>` from the data.
 
         Parameters
         ----------
@@ -370,7 +339,8 @@ class InstrumentGeometry:
 
         Attributes
         -------
-        fingering_chart: FingeringChart
+        fingering_chart: :py:class:`FingeringChart \
+        <openwind.technical.fingering_chart.FingeringChart>`
             The FingeringChart object containing the information on the
             fingerings.
 
@@ -411,7 +381,7 @@ class InstrumentGeometry:
         return FingeringChart(note_names, hole_labels, chart,
                               other_holes=missing)
 
-# %% Reading external files
+    # %% Reading external files
 
     @staticmethod
     def _interpret_data(data):
@@ -532,7 +502,7 @@ class InstrumentGeometry:
             raise ValueError("Invalid string for open/close: {}"
                              .format(s))
 
-# %% Creating DesignParameters / DesignShapes
+    # %% Creating DesignParameters / DesignShapes
 
     def __estim_value(self, param):
         """
@@ -581,14 +551,15 @@ class InstrumentGeometry:
         Create the DesignParameter associated to the format specified in the
         input parameter (except for holes parameters).
 
-        The subclass of `openwind.design.design_parameter.DesignParameter`
+        The subclass of :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>`
         instanciated depend of the format of `param`. The different format
         possible are (see the class docstring):
 
         - a float:      `x`                 => FixedParameter
-        - the string:   `'~x'`              => VariableParameter
-        - the string:   `'x_min<~x'`        => VariableParameterSquare
-        - the string:   `'x_min<~x<x_max'` => VariableParameterLimitedRange
+        - the string:   `'~x'`              => VariableParameter without bounds
+        - the string:   `'x_min<~x'`        => VariableParameter lower bounds
+        - the string:   `'x_min<~x<x_max'` => VariableParameter lower and upper bounds
 
         Parameters
         ----------
@@ -599,8 +570,10 @@ class InstrumentGeometry:
 
         Returns
         -------
-        openwind.design.design_parameter.DesignParameter
+        :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>`
             The DesignParameter object associated to the parameter
+
         """
         if isinstance(param, str):
             string_split = param.split(sep='<')
@@ -611,15 +584,15 @@ class InstrumentGeometry:
                 return FixedParameter(float(param), label)
             elif len(string_split) == 2 and string_split[1].startswith('~'):
                 min_value = float(string_split[0])
-                return VariableParameterSquare(float(string_split[1][1:]),
-                                               self.optim_params, min_value,
-                                               label)
+                return VariableParameter(float(string_split[1][1:]),
+                                               self.optim_params, label,
+                                               (min_value, np.inf))
             elif len(string_split) == 3 and string_split[1].startswith('~'):
-                authorized_range = [float(string_split[0]),
-                                    float(string_split[2])]
-                return VariableParameterLimitedRange(float(string_split[1][1:]),
-                                                     self.optim_params,
-                                                     authorized_range, label)
+                authorized_range = (float(string_split[0]),
+                                    float(string_split[2]))
+                return VariableParameter(float(string_split[1][1:]),
+                                                     self.optim_params, label,
+                                                     authorized_range)
             else:
                 raise ValueError(("the parameters format: '{:s}' is not "
                                   "recognized").format(param))
@@ -640,7 +613,8 @@ class InstrumentGeometry:
 
         Returns
         -------
-        openwind.design.design_shape.DesignShape
+        :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>`
             The design shape of the main bore part on which is placed the hole.
 
         """
@@ -659,7 +633,8 @@ class InstrumentGeometry:
         """
         Create the adequate DesignParameter for a hole position.
 
-        The subclass of `openwind.design.design_parameter.DesignParameter`
+        The subclass of :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>`
         instanciated depend of the format of `param`. The different format
         possible are (see the class docstring):
 
@@ -677,8 +652,10 @@ class InstrumentGeometry:
 
         Returns
         -------
-        openwind.design.design_parameter.DesignParameter
+        :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>`
             The DesignParameter object associated to the parameter
+
         """
         label = hole_label + '_position'
         if isinstance(x, str) and x.startswith('~') and x.endswith('%'):
@@ -692,7 +669,8 @@ class InstrumentGeometry:
         """
         Create the adequate DesignParameter for a hole radius.
 
-        The subclass of `openwind.design.design_parameter.DesignParameter`
+        The subclass of :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>`
         instanciated depend of the format of `r_data`. The different format
         possible are (see the class docstring):
 
@@ -712,8 +690,10 @@ class InstrumentGeometry:
 
         Returns
         -------
-        openwind.design.design_parameter.DesignParameter
+        :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>`
             The DesignParameter object associated to the parameter
+
         """
         label = hole_label + '_radius'
 
@@ -767,7 +747,7 @@ class InstrumentGeometry:
         return shape
 
 
-# %% Parsing geometry data
+    # %% Parsing geometry data
     def _parse_geometry(self, raw_part, Xlast, Rlast, label):
         """
         Interpret a list of data to design the main bore shape.
@@ -781,22 +761,27 @@ class InstrumentGeometry:
         ----------
         raw_part : list
             the list of the data.
-        Xlast : openwind.design.design_parameter.DesignParameter or None
+        Xlast : :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` or None
             The last ending position.
-        Rlast : openwind.design.design_parameter.DesignParameter or None
+        Rlast : :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` or None
             The last ending point radius.
         label : string
             The label of the shape.
 
         Returns
         -------
-        X : list of [ openwind.design.design_parameter.DesignParameter ]
+        X : list of [ :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` ]
             The 2 ends position of the shape.
-        R : list of [ openwind.design.design_parameter.DesignParameter ]
+        R : list of [ :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` ]
             The 2 ends radius of the shape.
         shape_type : string
             The type of the shape.
-        Geom_param : list of [ openwind.design.design_parameter.DesignParameter ]
+        Geom_param : list of [ :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` ]
             The eventual supplementary parameters (necessary for Circle,
             Bessel and Spline).
 
@@ -817,18 +802,22 @@ class InstrumentGeometry:
         ----------
         raw_part : list
             List of two data.
-        Xlast : openwind.design.design_parameter.DesignParameter or None
+        Xlast : :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` or None
             The last ending position.
-        Rlast : openwind.design.design_parameter.DesignParameter or None
+        Rlast : :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` or None
             The last ending point radius..
         label : string
             The label of the shape.
 
         Returns
         -------
-        X : list of [ openwind.design.design_parameter.DesignParameter ]
+        X : list of [ :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` ]
             The 2 ends position of the shape.
-        R : list of [ openwind.design.design_parameter.DesignParameter ]
+        R : list of [ :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` ]
             The 2 ends radius of the shape.
         shape_type : None
             Here it is the default type: 'linear'.
@@ -875,36 +864,42 @@ class InstrumentGeometry:
         ----------
         raw_part : list
             the list of the data (at list 4 elements: len(raw_part>=4)).
-        Xlast : openwind.design.design_parameter.DesignParameter or None
+        Xlast : :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` or None
             The last ending position. None for the first shape
-        Rlast : openwind.design.design_parameter.DesignParameter or None
+        Rlast : :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` or None
             The last ending point radius. None for the first shape
         label : string
             The label of the shape.
 
         Returns
         -------
-        X : list of [ openwind.design.design_parameter.DesignParameter ]
+        X : list of [ :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>`]
             The 2 ends position of the shape.
-        R : list of [ openwind.design.design_parameter.DesignParameter ]
+        R : list of [ :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` ]
             The 2 ends radius of the shape.
         shape_type : string
             The type of the shape.
-        Geom_param : list of [ openwind.design.design_parameter.DesignParameter ]
+        Geom_param : list of [ :py:class:`DesignParameter \
+        <openwind.design.design_parameter.DesignParameter>` ]
             The eventual supplementary parameters (necessary for Circle,
             Bessel and Spline).
+
         """
         if Xlast is None:  # For the first line, the first column is the begining of the main bore
             X = [self.__designparameter(raw_part[0], label + '_pos_minus')]
-        elif self.__estim_value(raw_part[0]) == Xlast.get_value():  # the parts of the main bore must be connected
+        elif np.isclose(self.__estim_value(raw_part[0]), Xlast.get_value()):  # the parts of the main bore must be connected
             X = [Xlast]
         else:
             msg = ('The main bore parts are not connected! '
                    'It is impossible to construct the instrument.')
             raise ValueError(msg)
 
-        if (Rlast is None
-            or self.__estim_value(raw_part[2]) != Rlast.get_value()):  # begining or discontinuity
+        if (Rlast is None or not
+            np.isclose(self.__estim_value(raw_part[2]), Rlast.get_value())):  # begining or discontinuity
             R = [self.__designparameter(raw_part[2], label + '_radius_minus')]
         else:
             R = [Rlast]
@@ -944,6 +939,7 @@ class InstrumentGeometry:
         ----------
         raw_holes : List
             the list of the data list for each hole.
+
         """
         warnings.warn("`position chimney radius type` hole file format is deprecated. "
                       "Use the file format with column headers instead.",
@@ -969,12 +965,12 @@ class InstrumentGeometry:
             - 'type' : the shape type of the chimney tube (optional), only
             'linear' is accepted.
 
-
         Parameters
         ----------
         raw_holes : List of [List]
             the list of the data list for each hole.
 
+
         """
         unifiate_column_name = {'x': 'position', 'position': 'position',
                                 'r': 'radius', 'radius': 'radius',
@@ -1038,9 +1034,18 @@ class InstrumentGeometry:
                                                      hole_position, label)
         shape_R = [r_param, r_param]
         new_shape = self._build_shape(shape_X, shape_R, 'linear', [])
-        self.holes.append(InstrumentGeometry.Hole(new_shape, hole_position, label))
+        self.holes.append(Hole(new_shape, hole_position, label))
 
-# %% Plots
+    # %% gets
+    def get_hole_labels(self):
+        """
+        Returns
+        -------
+        list
+            The labels of holes.
+
+        """
+        return [hole.label for hole in self.holes]
 
     def get_bore_list(self):
         """
@@ -1048,9 +1053,9 @@ class InstrumentGeometry:
 
         Returns
         -------
-        List
+        list
             The list of the data of the main bore.
-        List
+        list
             The list of data for the holes.
 
         """
@@ -1066,7 +1071,7 @@ class InstrumentGeometry:
 
     def get_main_bore_length(self):
         """
-        Return the total length of the main bore.
+        Return the total length of the main bore in meter.
 
         Returns
         -------
@@ -1079,7 +1084,7 @@ class InstrumentGeometry:
 
     def get_main_bore_radius_at(self, position):
         """
-        Return the radius of the main bore at given position.
+        Return the radius of the main bore at given position in meter.
 
         Parameters
         ----------
@@ -1100,8 +1105,10 @@ class InstrumentGeometry:
                 radius[is_in] = shape.get_radius_at(x_norm[is_in])
         return radius
 
+    # %% Plot
     def plot_InstrumentGeometry(self, figure=None, note=None, double_plot=True, **kwargs):
-        """Display the geometry and holes of the instrument.
+        """
+        Display the geometry and holes of the instrument.
 
         If a note name is given, also display the fingering of that note.
 
@@ -1115,10 +1122,12 @@ class InstrumentGeometry:
             By default all holes are outlined.
         kwargs:
             Additional arguments are passed to the `plt.plot` function.
+
         """
         if self.holes == []:
             double_plot = False
         # double_plot = (self.holes != [])
+        mmeter = 1e3
 
         if not figure:
             fig = plt.figure()
@@ -1135,11 +1144,11 @@ class InstrumentGeometry:
         for shape in self.main_bore_shapes:
             radius = np.append(shape.get_radius_at(x), np.nan)
             position = np.append(shape.get_position_from_xnorm(x), np.nan)
-            ax[0].plot(np.append(position, np.flip(position)),
-                     np.append(radius, np.flip(-radius)), **kwargs)
+            ax[0].plot(np.append(position, np.flip(position))*mmeter,
+                     np.append(radius, np.flip(-radius))*mmeter, **kwargs)
             if double_plot:
-                ax[1].plot(np.append(position, np.flip(position)),
-                         np.append(radius, np.flip(-radius)), **kwargs)
+                ax[1].plot(np.append(position, np.flip(position))*mmeter,
+                         np.append(radius, np.flip(-radius))*mmeter, **kwargs)
 
         if note:
             fingering = self.fingering_chart.fingering_of(note)
@@ -1161,31 +1170,38 @@ class InstrumentGeometry:
             main_radius = main_bore.get_radius_at(pos_norm)
             x_plot = position + np.append(radius, np.flip(-radius))
             y_plot = main_radius + np.append(chimney, np.flip(chimney))
-            hole_plot = plot_or_fill(ax[0], hole)(x_plot, y_plot, **kwargs)
+            hole_plot = plot_or_fill(ax[0], hole)(x_plot*mmeter, y_plot*mmeter, **kwargs)
             if type(hole_plot[0]).__name__ == 'Polygon':
                 hole_plot[0].set_edgecolor(hole_plot[0].get_facecolor())
 
             if double_plot:
                 theta = np.linspace(0,2*np.pi,100)
-                hole_plot = plot_or_fill(ax[1], hole)(position + np.mean(radius)*np.cos(theta),
-                                                      np.mean(radius)*np.sin(theta), **kwargs)
+                hole_plot = plot_or_fill(ax[1], hole)((position + np.mean(radius)*np.cos(theta))*mmeter,
+                                                      np.mean(radius)*np.sin(theta)*mmeter, **kwargs)
                 if type(hole_plot[0]).__name__ == 'Polygon':
                     hole_plot[0].set_edgecolor(hole_plot[0].get_facecolor())
-        ax[-1].set_xlabel('Position (m)')
-        ax[0].set_ylabel('Radius (m)')
+        ax[-1].set_xlabel('Position (mm)')
+        ax[0].set_ylabel('Radius (mm)')
         ax[0].axis('equal')
         if double_plot:
             ax[1].axis('equal')
-            ax[1].set_ylabel('Radius (m)')
-
-# %% print files
+            ax[1].set_ylabel('Radius (mm)')
 
+    # %% print files
     def print_files(self, generic_name, extension='.txt'):
         """
-        Print the files corresponding to this InstrumentGeometry.
+        .. deprecated:: 0.6.0
+            Replaced by :py:meth:`InstrumentGeometry.write_files()`
+        """
+        warnings.warn('This method is deprecated, please use write_files instead.')
+        self.write_files(generic_name, extension)
 
-        Print the three files (MainBore, Holes and FingeringChart) associated
-        to this maker model.
+    def write_files(self, generic_name, extension='.txt'):
+        """
+        Write the files corresponding to this InstrumentGeometry.
+
+        Write the three files (MainBore, Holes and FingeringChart) associated
+        to this instrument.
 
         Parameters
         ----------
@@ -1195,7 +1211,10 @@ class InstrumentGeometry:
             "generic_name_FingeringChart".
         extension : string, optional
             The extension used for the filenames. The default is '.txt'.
+
         """
+        if not extension.startswith('.'):
+            extension = '.' + extension
         if generic_name.endswith('.txt'):
             generic_name = generic_name[:-4]
         elif generic_name.endswith('.csv'):
@@ -1225,7 +1244,8 @@ class InstrumentGeometry:
             The string corresponding to the main bore shape.
 
         """
-        msg = '# x_0\t x_1\t r_0\t r_1\t type\t param\n'
+        msg = ('# {:<5s}\t{:<7s}\t{:<7s}\t{:<7s}\t{:<7s}\t{:<7s}'
+               '\n'.format('x_0', 'x_1', 'r_0', 'r_1', 'type', 'param'))
         for shape in self.main_bore_shapes:
             msg += '{}\n'.format(shape)
         if not filename:
