@@ -1,15 +1,24 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__author__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-              "Alexis Thibault", "Tobias Van Baarsel"]
-__copyright__ = "Copyright 2020, Inria"
-__credits__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-               "Alexis Thibault", "Tobias Van Baarsel"]
-__license__ = "GPL 3.0"
-__version__ = "0.4"
-__email__ = "openwind-contact@inria.fr"
-__status__ = "Dev"
+# Copyright (C) 2019-2021, INRIA
+#
+# This file is part of Openwind.
+#
+# Openwind is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Openwind is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Openwind.  If not, see <https://www.gnu.org/licenses/>.
+#
+# For more informations about authors, see the CONTRIBUTORS file
 
 
 import numpy as np
@@ -67,11 +76,11 @@ from numpy import pi
 #         not yet implemented
 
 
-# %% ============  COMPOSITE FUNCTIONS (PROBABLY MOST USED)  ============ %% #
+# %%  COMPOSITE FUNCTIONS (PROBABLY MOST USED)  %% #
 
 
 def gate(t1, t2, t3, t4, shape='fast', a=1):
-    '''
+    """
     On-Off gate between 0 and 'a' with fade-in and fade-out ramps
 
     Parameters
@@ -91,9 +100,10 @@ def gate(t1, t2, t3, t4, shape='fast', a=1):
 
     Returns
     -------
-     env(t) function, with a gate between t1 and t4, else 0
+    np.array :
+        env(t) function, with a gate between t1 and t4, else 0
 
-    '''
+    """
     if [t1, t2, t3, t4] != sorted([t1, t2, t3, t4]):
         warnings.warn('time arguments not increasing, ' +
                       ' might cause unexpected results')
@@ -129,7 +139,7 @@ def ADSR(t1, t2, amplitude,
          A, D, S, R,
          shape='fast',
          trem_a=0, trem_freq=5, trem_gate=gate(0, 0.1, 0.8, 1, 'slow')):
-    '''
+    """
     Attack-Decay-Sustain-Release envelope
 
     .. image:: /images/ADSR.png
@@ -163,9 +173,10 @@ def ADSR(t1, t2, amplitude,
 
     Returns
     -------
-    env(t) function returning the ADSR envelope when t1 < t < t2, else 0.
+    np.array :
+        env(t) function returning the ADSR envelope when t1 < t < t2, else 0.
 
-    '''
+    """
     if [t1, t2] != sorted([t1, t2]):
         warnings.warn('time arguments not increasing, ' +
                       ' might cause unexpected results')
@@ -322,14 +333,28 @@ def constant_with_initial_ramp(pm_max=5000, t_car=5e-2, t_down=1e10):
     return pm
 
 
-# %% ==================    SIMPLE FUNCTIONS    ================== %% #
+# %% SIMPLE FUNCTIONS  %% #
 
 # OLD : can easily be replaced by gate
 def triangle(pm_max=5000, duration=3.0):
     """
     Curve starting at zero, linearly increasing to pm_max until duration/2,
     and decreasing afterwards.
+
+    Parameters
+    ----------
+    pm_max : float, optional
+        The maximal value. The default is 5000.
+    duration : float, optional
+        The total duration. The default is 3.0.
+
+    Returns
+    -------
+    np.array
+        The triangle env(t).
+
     """
+
     half_dur = duration / 2
 
     def pm(t):
@@ -342,7 +367,7 @@ def triangle(pm_max=5000, duration=3.0):
 
 
 def bump(A, t0, width):
-    '''
+    """
     C-inf bump function, with zero value and derivative at t0 +/- time_scale/2
 
     Parameters
@@ -356,10 +381,11 @@ def bump(A, t0, width):
 
     Returns
     -------
-    env(t) function, with bump when
-        t0-width/2 < t < t0+width/2, else 0.
+    np.array :
+        env(t) function, with bump when :math:`t0-width/2 < t < t0+width/2`,
+        else 0.
 
-    '''
+    """
     def env(t):
         with np.errstate(all='ignore'):
             return A * ((np.nan_to_num(np.exp(-1 / (1 - abs((t - t0) /
@@ -376,11 +402,11 @@ def from_file():  # not implemented yet
     return 0
 
 
-# %% =========  BASIC FUNCTIONS (PROBABLY NOT FOR DIRECT USE)  ========= %% #
+# %% BASIC FUNCTIONS (PROBABLY NOT FOR DIRECT USE) %% #
 
 
 def fade(t1, x1, t2, x2, shape='fast'):
-    '''
+    """
     technically half of bump function :
         useful for C-inf ramp between two values
 
@@ -399,9 +425,10 @@ def fade(t1, x1, t2, x2, shape='fast'):
 
     Returns
     -------
-    env(t) function, with C-inf ramp from x1 to x2 when t1 < t < t2, else 0.
+    np.array :
+        env(t) function, with C-inf ramp from x1 to x2 when t1 < t < t2, else 0.
 
-    '''
+    """
 
     if [t1, t2] != sorted([t1, t2]):
         warnings.warn('time arguments not increasing, ' +
@@ -444,7 +471,7 @@ def fade(t1, x1, t2, x2, shape='fast'):
 
 
 def exp(t1, x1, t2, x2):
-    '''
+    """
     piece of exponential function between two points
 
     Parameters
@@ -460,9 +487,10 @@ def exp(t1, x1, t2, x2):
 
     Returns
     -------
-    env(t) function, with exponential when t1 < t < t2, else 0.
+    np.array :
+        env(t) function, with exponential when t1 < t < t2, else 0.
 
-    '''
+    """
     if [t1, t2] != sorted([t1, t2]):
         warnings.warn('time arguments not increasing, ' +
                       ' might cause unexpected results')
@@ -474,7 +502,7 @@ def exp(t1, x1, t2, x2):
 
 
 def cos(t1, x1, t2, x2):
-    '''
+    """
 
 
     Parameters
@@ -490,9 +518,10 @@ def cos(t1, x1, t2, x2):
 
     Returns
     -------
-    env(t) function, with cosine ramp from x1 to x2 when t1 < t < t2, else 0.
+    np.array :
+        env(t) function, with cosine ramp from x1 to x2 when t1 < t < t2, else 0.
 
-    '''
+    """
     if [t1, t2] != sorted([t1, t2]):
         warnings.warn('time arguments not increasing, ' +
                       ' might cause unexpected results')
@@ -504,7 +533,7 @@ def cos(t1, x1, t2, x2):
 
 
 def tremolo(A, frequency, phase=0):
-    '''
+    """
     sine oscillation of given amplitude, frequency and phase
 
     Parameters
@@ -518,9 +547,10 @@ def tremolo(A, frequency, phase=0):
 
     Returns
     -------
-    env(t) function, oscillation on the whole range of t
+    np.array :
+        env(t) function, oscillation on the whole range of t
 
-    '''
+    """
     def env(t):
         return A * np.sin(2 * np.pi * frequency * t + phase)
 
@@ -528,7 +558,7 @@ def tremolo(A, frequency, phase=0):
 
 
 def constant(t1, t2, C):
-    '''
+    """
 
     constant value over the range [t1, t2], else 0
 
@@ -543,9 +573,10 @@ def constant(t1, t2, C):
 
     Returns
     -------
-    env(t) function, equals C over the range [t1, t2], else 0
+    np.array :
+        env(t) function, equals C over the range [t1, t2], else 0
 
-    '''
+    """
     if [t1, t2] != sorted([t1, t2]):
         warnings.warn('time arguments not increasing, ' +
                       ' might cause unexpected results')
@@ -559,7 +590,7 @@ def constant(t1, t2, C):
 
 
 def ramp(t1, x1, t2, x2):
-    '''
+    """
 
     linear ramp x = a*t + b between two points defined by (t, x)
 
@@ -576,9 +607,10 @@ def ramp(t1, x1, t2, x2):
 
     Returns
     -------
-    env(t) function returning the ramp when t1 < t < t2, else 0.
+    np.array :
+        env(t) function returning the ramp when t1 < t < t2, else 0.
 
-    '''
+    """
     if [t1, t2] != sorted([t1, t2]):
         warnings.warn('time arguments not increasing, ' +
                       ' might cause unexpected results')
