@@ -1,15 +1,24 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__author__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-              "Alexis Thibault", "Tobias Van Baarsel"]
-__copyright__ = "Copyright 2020, Inria"
-__credits__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-               "Alexis Thibault", "Tobias Van Baarsel"]
-__license__ = "GPL 3.0"
-__version__ = "0.4"
-__email__ = "openwind-contact@inria.fr"
-__status__ = "Dev"
+# Copyright (C) 2019-2021, INRIA
+#
+# This file is part of Openwind.
+#
+# Openwind is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Openwind is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Openwind.  If not, see <https://www.gnu.org/licenses/>.
+#
+# For more informations about authors, see the CONTRIBUTORS file
 
 """
 Module for TemporalSolver class.
@@ -20,36 +29,45 @@ from openwind.continuous import (ThermoviscousLossless,
                                  RadiationPade,
                                  PhysicalRadiation,
                                  JunctionTjoint, JunctionSimple,
-                                 Valve, Flow,
+                                 JunctionDiscontinuity,
+                                 Reed1dof, Flow,
                                  RadiationPerfectlyOpen
                                  )
 from openwind.temporal import (TemporalPipe, TemporalRadiation,
                                TemporalJunction, TemporalLossyPipe,
                                TemporalSimpleJunction,
+                               TemporalJunctionDiscontinuity,
                                TemporalFlowCondition,
-                               TemporalValve,
+                               TemporalReed1dof,
                                TemporalPressureCondition
                                )
 from openwind.temporal import ExecuteScore
 from openwind.tracker import SimulationTracker
 import numpy as np
-import pdb
+
 
 class TemporalSolver:
     """Prepare an instrument for simulation in time domain.
 
     This class is responsible for converting a continuous instrument model
-    (openwind.continuous.InstrumentPhysics) to objects that can run the
-    numerical schemes, and for running said scheme.
+    (:py:class:`InstrumentPhysics<openwind.continuous.instrument_physics.InstrumentPhysics>`)
+    to objects that can run the numerical schemes, and for running said scheme.
 
     Parameters
     ----------
-    instru_physics : openwind.continuous.instrument_physics.IntrumentPhysics
+    instru_physics : :py:class:`InstrumentPhysics<openwind.continuous.instrument_physics.InstrumentPhysics>`
         Description of the instrument.
-    cfl_alpha: float
-        `dt` will be set to `cfl_alpha * dt_max` or slightly lower.
-    **discr_params :
-        Discretization parameters. See openwind.discretization.mesh .
+    cfl_alpha: float, optional
+        Coefficient used to guarantee the respect of the CFL condition.
+        `dt` will be set to `cfl_alpha * dt_max` or slightly lower. It must be
+        in ]0,1]. Default is 0.9.
+    theta_scheme_parameter : float, optional
+        Coefficient used in the excitator scheme. See
+        :py:class:`TemporalReed1dof <openwind.temporal.treed1dof.TemporalReed1dof>`.
+        Default is 0.25.
+    **discr_params : keywords
+        Discretization parameters. See :py:class:`Mesh<openwind.discretization.mesh>`.
+
     """
 
     def __init__(self, instru_physics,
@@ -128,14 +146,14 @@ class TemporalSolver:
 
     def _convert_connector(self, connector, ends):
         """
-        Construct the appropriate temporal version of a component.
+        Construct the appropriate temporal version of a connector.
 
         Parameters
         ----------
-        component : openwind.continuous.NetlistComponent
+        connector : :py:class:`NetlistConnector<openwind.continuous.netlist.NetlistConnector>`
             Continuous model for radiation, junction, or source.
         ends : List[TPipe.End]
-            The list of all `TPipe.End`s this component connects to.
+            The list of all `TPipe.End`s this connector connects to.
 
         Returns
         -------
@@ -144,7 +162,7 @@ class TemporalSolver:
         """
 
         if isinstance(connector, PhysicalRadiation):
-            radiation_model = connector.model
+            radiation_model = connector
             if isinstance(radiation_model, RadiationPerfectlyOpen):
                 return TemporalPressureCondition(connector, ends,
                                                  t_solver=self)
@@ -162,8 +180,12 @@ class TemporalSolver:
         if isinstance(connector, JunctionSimple):
             return TemporalSimpleJunction(connector, ends,
                                           t_solver=self)
-        if isinstance(connector, Valve):
-            return TemporalValve(connector, ends, t_solver=self,
+        if isinstance(connector, JunctionDiscontinuity):
+            return TemporalJunctionDiscontinuity(connector, ends,
+                                                 t_solver=self)
+
+        if isinstance(connector, Reed1dof):
+            return TemporalReed1dof(connector, ends, t_solver=self,
                                 theta=self.theta_scheme_parameter)
         if isinstance(connector, Flow):
             return TemporalFlowCondition(connector, ends, t_solver=self)
@@ -188,7 +210,12 @@ class TemporalSolver:
             t_connector.set_dt(self._dt)
 
     def get_dt(self):
-        """Time step duration."""
+        """
+        Returns
+        -------
+        float
+            Time step duration.
+        """
         return self._dt
 
     def reset(self):
@@ -198,7 +225,12 @@ class TemporalSolver:
         self._current_time = 0.0
 
     def get_current_time(self):
-        """Current in-simulation physical time in seconds."""
+        """
+        Returns
+        -------
+        float
+            Current in-simulation physical time in seconds.
+        """
         return self._current_time
 
     def one_step(self, check_scheme=False):
@@ -206,7 +238,7 @@ class TemporalSolver:
 
         See also
         --------
-        run_simulation
+        :py:meth:`run_simulation()<TemporalSolver.run_simulation>`
         """
         # We consider that during the update, current time is (n+1/2)*dt.
         self._current_time += self._dt/2
@@ -222,11 +254,24 @@ class TemporalSolver:
         self._current_time += self._dt/2
 
     def energy(self):
-        """Calculate total numerical energy stored in instrument."""
+        """Calculate total numerical energy stored in instrument.
+
+        Returns
+        -------
+        float
+            The total energy stored
+        """
         return sum(t_comp.energy() for t_comp in self.t_components)
 
     def dissipated_last_step(self):
-        """Calculate total numerical energy dissipated during last step."""
+        """
+        Calculate total numerical energy dissipated during last step.
+
+        Returns
+        -------
+        float
+            The total dissipated energy
+        """
         return sum(t_comp.dissipated_last_step() for t_comp
                    in self.t_components)
 
@@ -253,21 +298,22 @@ class TemporalSolver:
             as an argument.
         enable_tracker_display : bool, optional
             Whether to enable printing information on percentage of progression
-            and remaining time. Default is `True`. See openwind.tracker .
+            and remaining time. Default is `True`. See
+            :py:class:`SimulationTracker<openwind.tracker.SimulationTracker>`.
         energy_check : bool, optional
             Whether to check that the scheme is energy-consistent. More costly.
-            See EnergyCheck.
+            See :py:class:`EnergyCheck<EnergyCheck>`. Default is False
         n_steps : int, optional
             If given, forces the simulation to run in exactly `n_steps` steps.
-            Fails if that contradicts stability.
+            Fails if that contradicts stability. Default is None.
 
         See also
         --------
-        run_simulation_steps:
+        :py:meth:`run_simulation_steps`:
             To run for a given number of steps instead.
         """
         self._execute_score.set_score(self.instru_physics.player.get_score())
-        #pdb.set_trace()
+        self.instru_physics._update_player()
         n_steps_needed = int(np.ceil(duration / self._dt))
         if n_steps is not None:
             print("Custom number of steps:", n_steps)
@@ -289,6 +335,21 @@ class TemporalSolver:
 
         Does not change dt.
 
+        Parameters
+        ----------
+        n_steps : int
+            The number of steps
+        callback : callable, optional
+            A function to call after each step, taking this TemporalSolver
+            as an argument.
+        enable_tracker_display : bool, optional
+            Whether to enable printing information on percentage of progression
+            and remaining time. Default is `True`. See
+            :py:class:`SimulationTracker<openwind.tracker.SimulationTracker>`.
+        energy_check : bool, optional
+            Whether to check that the scheme is energy-consistent. More costly.
+            See :py:class:`EnergyCheck<EnergyCheck>`. Default is False
+
         See also
         --------
         run_simulation:
@@ -317,12 +378,30 @@ class TemporalSolver:
             self.energy_check.finish()
 
     def get_lengths_pipes(self):
+        """
+        Returns
+        -------
+        list of float
+            The length of each pipe (in meter)
+        """
         return [t_pipe.pipe.get_length() for t_pipe in self.t_pipes]
 
     def get_orders_mesh(self):
+        """
+        Returns
+        -------
+        list of list of int
+            The order of each elements of each pipe
+        """
         return [t_pipe.mesh.get_orders().tolist() for t_pipe in self.t_pipes]
 
     def get_elements_mesh(self):
+        """
+        Returns
+        -------
+        list of int
+            The number of elements on each pipe.
+        """
         return [len(x) for x in self.get_orders_mesh()]
 
     def __get_mesh_info(self):
@@ -341,6 +420,17 @@ class TemporalSolver:
         return msg
 
     def discretization_infos(self):
+        """
+        Information of the total mesh used to solve the problem.
+
+        See Also
+        --------
+        :py:class:`Mesh <openwind.discretization.mesh.Mesh>`
+
+        Returns
+        -------
+        str
+        """
         print(self.__get_mesh_info())
 
 
@@ -354,8 +444,15 @@ class EnergyCheck:
     --------
     >>> t_solver.run_simulation(duration=0.1, energy_check=True)
 
-    WARNING : Does not take energy sources into account (yet). Will fail if
-    a component is a source of energy (for instance a nonzero TFlowCondition).
+    .. warning::
+        Does not take energy sources into account (yet). Will fail if
+        a component is a source of energy (for instance a nonzero
+        :py:class:`TemporalFlowCondition<openwind.temporal.tflow_condition.TemporalFlowCondition>`).
+
+    Parameters
+    ----------
+    t_solver : :py:class:`TemporalSolver<TemporalSolver>`
+        The associated solver
     """
 
     def __init__(self, t_solver):
@@ -373,16 +470,13 @@ class EnergyCheck:
         self.energy = self.t_solver.energy()
         dissip_last_step = self.t_solver.dissipated_last_step()
         self.dissipated_total += dissip_last_step
-        #print(self.energy)
-        #print(self.t_solver.get_dt())
+
         residual = (self.energy - self.prev_energy + dissip_last_step) \
             / (self.t_solver.get_dt() * self.energy)
-        #print(residual)
+
         err = abs(residual)
         self.energy_errs.append(residual)
 
-        # print("t = {:.3e} ; energy = {} ; dissipated_total = {} ; energy+dissip = {} ; error on energy balance = {}".format(self.t_solver.get_current_time(),
-        #       self.energy, self.dissipated_total, self.energy+self.dissipated_total, err))
         if err > self.max_err:
             self.max_err = err
         if err > 1:         # When huge error, stop the program
